@@ -1,10 +1,11 @@
-# Copyright (C) 2022 CVAT.ai Corporation
+# Copyright (C) CVAT.ai Corporation
 #
 # SPDX-License-Identifier: MIT
 
 from __future__ import annotations
 
 from cvat_sdk.api_client import apis, models
+from cvat_sdk.core.helpers import get_paginated_collection
 from cvat_sdk.core.proxies.model_proxy import (
     ModelCreateMixin,
     ModelDeleteMixin,
@@ -50,6 +51,14 @@ class Issue(
 ):
     _model_partial_update_arg = "patched_issue_write_request"
 
+    def get_comments(self) -> list[Comment]:
+        return [
+            Comment(self._client, m)
+            for m in get_paginated_collection(
+                self._client.api_client.comments_api.list_endpoint, issue_id=self.id
+            )
+        ]
+
 
 class IssuesRepo(
     _IssueRepoBase,
